@@ -32,6 +32,7 @@ PCAPlot <- function(object, ...) {
 
 #' @rdname SpatialPlot
 #' @concept convenience
+#' @concept spatial
 #' @export
 #'
 SpatialDimPlot <- function(
@@ -51,6 +52,7 @@ SpatialDimPlot <- function(
   combine = TRUE,
   pt.size.factor = 1.6,
   alpha = c(1, 1),
+  image.alpha = 1,
   stroke = 0.25,
   label.box = TRUE,
   interactive = FALSE,
@@ -73,6 +75,7 @@ SpatialDimPlot <- function(
     combine = combine,
     pt.size.factor = pt.size.factor,
     alpha = alpha,
+    image.alpha = image.alpha,
     stroke = stroke,
     label.box = label.box,
     interactive = interactive,
@@ -82,6 +85,7 @@ SpatialDimPlot <- function(
 
 #' @rdname SpatialPlot
 #' @concept convenience
+#' @concept spatial
 #' @export
 #'
 SpatialFeaturePlot <- function(
@@ -96,6 +100,7 @@ SpatialFeaturePlot <- function(
   combine = TRUE,
   pt.size.factor = 1.6,
   alpha = c(1, 1),
+  image.alpha = 1,
   stroke = 0.25,
   interactive = FALSE,
   information = NULL
@@ -112,6 +117,7 @@ SpatialFeaturePlot <- function(
     combine = combine,
     pt.size.factor = pt.size.factor,
     alpha = alpha,
+    image.alpha = image.alpha,
     stroke = stroke,
     interactive = interactive,
     information = information
@@ -169,3 +175,44 @@ SpecificDimPlot <- function(object, ...) {
     }
   )
 }
+
+#' Read output from Parse Biosciences
+#'
+#' @param data.dir Directory containing the data files
+#' @param ... Extra parameters passed to \code{\link{ReadMtx}}
+#' @concept convenience
+#' @export
+#'
+ReadParseBio <- function(data.dir, ...) {
+  mtx <- file.path(data.dir, "DGE.mtx")
+  cells <- file.path(data.dir, "cell_metadata.csv")
+  features <- file.path(data.dir, "all_genes.csv")
+  return(ReadMtx(
+    mtx = mtx,
+    cells = cells,
+    features = features,
+    cell.column = 1,
+    feature.column = 2,
+    cell.sep = ",",
+    feature.sep = ",",
+    skip.cell = 1,
+    skip.feature = 1,
+    mtx.transpose = TRUE
+  ))
+}
+
+#' Read output from STARsolo
+#'
+#' @param data.dir Directory containing the data files
+#' @param ... Extra parameters passed to \code{\link{ReadMtx}}
+#'
+#' @rdname ReadSTARsolo
+#' @concept convenience
+#' @export
+#'
+ReadSTARsolo <- function(data.dir, ... ) {
+  mtx <- file.path(data.dir, "matrix.mtx")
+  cells <- file.path(data.dir, "barcodes.tsv")
+  features <- file.path(data.dir, "features.tsv")
+  return(ReadMtx(mtx = mtx, cells = cells, features = features, ...))
+}
