@@ -335,7 +335,7 @@ ProjectUMAP.default <- function(
   query.dims = NULL,
   reference,
   reference.dims = NULL,
-  k.param = 20,
+  k.param = 30,
   nn.method = "annoy",
   n.trees = 50,
   annoy.metric = "cosine",
@@ -389,7 +389,7 @@ ProjectUMAP.DimReduc <- function(
   query.dims = NULL,
   reference,
   reference.dims = NULL,
-  k.param = 20,
+  k.param = 30,
   nn.method = "annoy",
   n.trees = 50,
   annoy.metric = "cosine",
@@ -439,7 +439,7 @@ ProjectUMAP.Seurat <- function(
   reference,
   reference.reduction,
   reference.dims = NULL,
-  k.param = 20,
+  k.param = 30,
   nn.method = "annoy",
   n.trees = 50,
   annoy.metric = "cosine",
@@ -479,6 +479,7 @@ ProjectUMAP.Seurat <- function(
     neighbor.name = neighbor.name,
     reduction.model = reference[[reduction.model]],
     reduction.key = reduction.key,
+    assay = DefaultAssay(query),
     ...
   )
   query[[reduction.name]] <- proj.umap$proj.umap
@@ -635,6 +636,9 @@ RunCCA.Seurat <- function(
     merge.data = TRUE,
     ...
   )
+  rownames(x = cca.results$ccv) <- Cells(x = combined.object)
+  colnames(x = data1) <- Cells(x = combined.object)[1:ncol(x = data1)]
+  colnames(x = data2) <- Cells(x = combined.object)[(ncol(x = data1) + 1):length(x = Cells(x = combined.object))]
   combined.object[['cca']] <- CreateDimReducObject(
     embeddings = cca.results$ccv[colnames(combined.object), ],
     assay = assay1,
@@ -1210,6 +1214,10 @@ RunUMAP.default <- function(
   seed.use = 42,
   metric.kwds = NULL,
   angular.rp.forest = FALSE,
+  densmap = FALSE,
+  dens.lambda = 2,
+  dens.frac = 0.3,
+  dens.var.shift = 0.1,
   verbose = TRUE,
   ...
 ) {
@@ -1232,6 +1240,10 @@ RunUMAP.default <- function(
     umap.method <- "uwot"
     return.model <- TRUE
   }
+  if (densmap && umap.method != 'umap-learn'){
+    warning("densmap is only supported by umap-learn method. Method is changed to 'umap-learn'")
+    umap.method <- 'umap-learn'
+  }
   if (return.model) {
     if (verbose) {
       message("UMAP will return its model")
@@ -1254,6 +1266,9 @@ RunUMAP.default <- function(
       if (!py_module_available(module = 'umap')) {
         stop("Cannot find UMAP, please install through pip (e.g. pip install umap-learn).")
       }
+      if (!py_module_available(module = 'sklearn')) {
+        stop("Cannot find sklearn, please install through pip (e.g. pip install scikit-learn).")
+      }
       if (!is.null(x = seed.use)) {
         py_set_seed(seed = seed.use)
       }
@@ -1261,7 +1276,14 @@ RunUMAP.default <- function(
         n.epochs <- as.integer(x = n.epochs)
       }
       umap_import <- import(module = "umap", delay_load = TRUE)
-      umap <- umap_import$UMAP(
+      sklearn <- import("sklearn", delay_load = TRUE)
+      if (densmap &&
+          numeric_version(x = umap_import$pkg_resources$get_distribution("umap-learn")$version) <
+          numeric_version(x = "0.5.0")) {
+        stop("densmap is only supported by versions >= 0.5.0 of umap-learn. Upgrade umap-learn (e.g. pip install --upgrade umap-learn).")
+      }
+      random.state <- sklearn$utils$check_random_state(seed = as.integer(x = seed.use))
+      umap.args <- list(
         n_neighbors = as.integer(x = n.neighbors),
         n_components = as.integer(x = n.components),
         metric = metric,
@@ -1273,23 +1295,27 @@ RunUMAP.default <- function(
         local_connectivity = local.connectivity,
         repulsion_strength = repulsion.strength,
         negative_sample_rate = negative.sample.rate,
+        random_state = random.state,
         a = a,
         b = b,
         metric_kwds = metric.kwds,
         angular_rp_forest = angular.rp.forest,
         verbose = verbose
       )
+      if (numeric_version(x = umap_import$pkg_resources$get_distribution("umap-learn")$version) >=
+          numeric_version(x = "0.5.0")) {
+        umap.args <- c(umap.args, list(
+          densmap = densmap,
+          dens_lambda = dens.lambda,
+          dens_frac = dens.frac,
+          dens_var_shift = dens.var.shift,
+          output_dens = FALSE
+        ))
+      }
+      umap <- do.call(what = umap_import$UMAP, args = umap.args)
       umap$fit_transform(as.matrix(x = object))
     },
     'uwot' = {
-      if (metric == 'correlation') {
-        warning(
-          "UWOT does not implement the correlation metric, using cosine instead",
-          call. = FALSE,
-          immediate. = TRUE
-        )
-        metric <- 'cosine'
-      }
       if (is.list(x = object)) {
         umap(
           X = NULL,
@@ -1360,7 +1386,12 @@ RunUMAP.default <- function(
         )
       }
       if (is.list(x = object)) {
-        uwot::umap_transform(
+        if (ncol(object$idx) != model$n_neighbors) {
+          warning("Number of neighbors between query and reference ", 
+          "is not equal to the number of neighbros within reference")
+          model$n_neighbors <- ncol(object$idx)
+        }
+       umap_transform(
           X = NULL,
           nn_method = object,
           model = model,
@@ -1429,11 +1460,13 @@ RunUMAP.Graph <- function(
   uwot.sgd = FALSE,
   seed.use = 42L,
   metric.kwds = NULL,
+  densmap = FALSE,
+  densmap.kwds = NULL,
   verbose = TRUE,
   reduction.key = 'UMAP_',
   ...
 ) {
-  CheckDots(...)
+  #CheckDots(...)
   if (umap.method != 'umap-learn') {
     warning(
       "Running UMAP on Graph objects is only supported using the umap-learn method",
@@ -1465,7 +1498,7 @@ RunUMAP.Graph <- function(
   b <- b %||% ab.params[[2]]
   n.epochs <- n.epochs %||% 0L
   random.state <- sklearn$utils$check_random_state(seed = as.integer(x = seed.use))
-  embeddings <- umap$umap_$simplicial_set_embedding(
+  umap.args <- list(
     data = data,
     graph = object,
     n_components = n.components,
@@ -1481,6 +1514,18 @@ RunUMAP.Graph <- function(
     metric_kwds = metric.kwds,
     verbose = verbose
   )
+  if (numeric_version(x = umap$pkg_resources$get_distribution("umap-learn")$version) >=
+      numeric_version(x = "0.5.0")) {
+    umap.args <- c(umap.args, list(
+      densmap = densmap,
+      densmap_kwds = densmap.kwds,
+      output_dens = FALSE
+    ))
+  }
+  embeddings <- do.call(what = umap$umap_$simplicial_set_embedding, args = umap.args)
+  if (length(x = embeddings) == 2) {
+    embeddings <- embeddings[[1]]
+  }
   rownames(x = embeddings) <- colnames(x = data)
   colnames(x = embeddings) <- paste0("UMAP_", 1:n.components)
   # center the embeddings on zero
@@ -1578,6 +1623,26 @@ RunUMAP.Neighbor <- function(
 #' approximate nearest neighbor search. This can be faster, but is mostly on useful for metric that
 #' use an angular style distance such as cosine, correlation etc. In the case of those metrics
 #' angular forests will be chosen automatically.
+#' @param densmap Whether to use the density-augmented objective of densMAP.
+#' Turning on this option generates an embedding where the local densities
+#' are encouraged to be correlated with those in the original space.
+#' Parameters below with the prefix ‘dens’ further control the behavior
+#' of this extension. Default is FALSE. Only compatible with 'umap-learn' method
+#' and version of umap-learn >= 0.5.0
+#' @param densmap.kwds A dictionary of arguments to pass on to the densMAP optimization.
+#' @param dens.lambda Specific parameter which controls the regularization weight
+#' of the density correlation term in densMAP. Higher values prioritize density
+#' preservation over the UMAP objective, and vice versa for values closer to zero.
+#' Setting this parameter to zero is equivalent to running the original UMAP algorithm.
+#' Default value is 2.
+#' @param dens.frac Specific parameter which controls the fraction of epochs
+#' (between 0 and 1) where the density-augmented objective is used in densMAP.
+#' The first (1 - dens_frac) fraction of epochs optimize the original UMAP
+#' objective before introducing the density correlation term. Default is 0.3.
+#' @param dens.var.shift Specific parameter which specifies a small constant
+#' added to the variance of local radii in the embedding when calculating
+#' the density correlation objective to prevent numerical instability from
+#' dividing by a small number. Default is 0.1.
 #' @param reduction.name Name to store dimensional reduction under in the Seurat object
 #' @param reduction.key dimensional reduction key, specifies the string before
 #' the number for the dimension names. UMAP by default
@@ -1620,6 +1685,10 @@ RunUMAP.Seurat <- function(
   seed.use = 42L,
   metric.kwds = NULL,
   angular.rp.forest = FALSE,
+  densmap = FALSE,
+  dens.lambda = 2,
+  dens.frac = 0.3,
+  dens.var.shift = 0.1,
   verbose = TRUE,
   reduction.name = 'umap',
   reduction.key = 'UMAP_',
@@ -1654,9 +1723,27 @@ RunUMAP.Seurat <- function(
         call. = FALSE
       )
     }
-  }  else if (!is.null( x = nn.name)){
+  }  else if (!is.null(x = nn.name)) {
+    if (!inherits(x = object[[nn.name]], what = "Neighbor")) {
+      stop(
+        "Please specify a Neighbor object name, ",
+        "instead of the name of a ",
+        class(object[[nn.name]]),
+        " object",
+        call. = FALSE
+      )
+    }
     data.use <- object[[nn.name]]
   } else if (!is.null(x = graph)) {
+    if (!inherits(x = object[[graph]], what = "Graph")) {
+      stop(
+        "Please specify a Graph object name, ",
+        "instead of the name of a ",
+        class(object[[graph]]),
+        " object",
+        call. = FALSE
+      )
+    }
     data.use <- object[[graph]]
   } else {
     stop("Please specify one of dims, features, or graph")
@@ -1684,6 +1771,10 @@ RunUMAP.Seurat <- function(
     seed.use = seed.use,
     metric.kwds = metric.kwds,
     angular.rp.forest = angular.rp.forest,
+    densmap = densmap,
+    dens.lambda = dens.lambda,
+    dens.frac = dens.frac,
+    dens.var.shift = dens.var.shift,
     reduction.key = reduction.key,
     verbose = verbose
   )
@@ -1746,7 +1837,7 @@ ScoreJackStraw.DimReduc <- function(object, dims = 1:5, score.thresh = 1e-5, ...
   JS(object = object) <- ScoreJackStraw(
     object = JS(object = object),
     dims = dims,
-    score.thresh = 1e-5,
+    score.thresh = score.thresh,
     ...
   )
   return(object)
@@ -1774,6 +1865,7 @@ ScoreJackStraw.Seurat <- function(
   object[[reduction]] <- ScoreJackStraw(
     object = object[[reduction]],
     dims = dims,
+    score.thresh = score.thresh,
     ...
   )
   if (do.plot) {
@@ -2179,13 +2271,14 @@ L2Norm <- function(vec) {
 PrepDR <- function(
   object,
   features = NULL,
+  slot = 'scale.data',
   verbose = TRUE
 ) {
   if (length(x = VariableFeatures(object = object)) == 0 && is.null(x = features)) {
     stop("Variable features haven't been set. Run FindVariableFeatures() or provide a vector of feature names.")
   }
-  data.use <- GetAssayData(object = object, slot = "scale.data")
-  if (nrow(x = data.use ) == 0) {
+  data.use <- GetAssayData(object = object, slot = slot)
+  if (nrow(x = data.use ) == 0 && slot == "scale.data") {
     stop("Data has not been scaled. Please run ScaleData and retry")
   }
   features <- features %||% VariableFeatures(object = object)
@@ -2197,7 +2290,13 @@ PrepDR <- function(
     }
   }
   features <- features.keep
-  features.var <- apply(X = data.use[features, ], MARGIN = 1, FUN = var)
+
+  if (inherits(x = data.use, what = 'dgCMatrix')) {
+    features.var <- RowVarSparse(mat = data.use[features, ])
+  }
+  else {
+    features.var <- RowVar(x = data.use[features, ])
+  }
   features.keep <- features[features.var > 0]
   if (length(x = features.keep) < length(x = features)) {
     features.exclude <- setdiff(x = features, y = features.keep)
@@ -2223,6 +2322,7 @@ PrepDR <- function(
 #'
 #' @importFrom irlba irlba
 #'
+#' @concept dimensional_reduction
 #' @rdname RunSPCA
 #' @export
 RunSPCA.default <- function(
@@ -2264,6 +2364,7 @@ RunSPCA.default <- function(
 #' using the variable features for the Assay.
 #'
 #' @rdname RunSPCA
+#' @concept dimensional_reduction
 #' @export
 #' @method RunSPCA Assay
 #'
@@ -2298,6 +2399,7 @@ RunSPCA.Assay <- function(
 
 #' @param reduction.name dimensional reduction name, spca by default
 #' @rdname RunSPCA
+#' @concept dimensional_reduction
 #' @export
 #' @method RunSPCA Seurat
 #'
@@ -2336,3 +2438,143 @@ RunSPCA.Seurat <- function(
   object <- LogSeuratCommand(object = object)
   return(object)
 }
+
+#' @param assay Name of Assay SLSI is being run on
+#' @param n Total Number of SLSI components to compute and store
+#' @param verbose Display messages
+#' @param reduction.key dimensional reduction key, specifies the string before
+#' the number for the dimension names
+#' @param graph Graph used supervised by SLSI
+#' @param seed.use Set a random seed. Setting NULL will not set a seed.
+#'
+#' @importFrom irlba irlba
+#' @importMethodsFrom Matrix t
+#'
+#' @concept dimensional_reduction
+#' @rdname RunSLSI
+#' @export
+RunSLSI.default <- function(
+  object,
+  assay = NULL,
+  n = 50,
+  reduction.key = "SLSI_",
+  graph = NULL,
+  verbose = TRUE,
+  seed.use = 42,
+  ...
+) {
+  if (!is.null(x = seed.use)) {
+    set.seed(seed = seed.use)
+  }
+  n <- min(n, nrow(x = object) - 1)
+
+  if (verbose) {
+    message("Smoothing peaks matrix")
+  }
+  object.smooth <- t(x = graph) %*% (t(x = object) %*% object) %*% graph
+  if (verbose) {
+    message("Performing eigendecomposition")
+  }
+  svd.V <- irlba(A = object.smooth, nv = n, nu = n, ...)
+  sigma <- sqrt(x = svd.V$d)
+  feature.loadings <- object %*% (graph %*% svd.V$u) %*% diag(x = 1/sigma)
+  feature.loadings <- as.matrix(x = feature.loadings)
+  cell.embeddings <- t(x = object) %*% feature.loadings %*% diag(x = 1/sigma)
+  cell.embeddings <- as.matrix(x = cell.embeddings)
+
+  # construct svd list stored in misc for LSI projection
+  svd.lsi <- svd.V
+  svd.lsi$d <- sigma
+  svd.lsi$u <- feature.loadings
+  svd.lsi$v <- cell.embeddings
+
+  colnames(x = cell.embeddings) <- paste0(reduction.key, 1:ncol(cell.embeddings))
+  reduction.data <- CreateDimReducObject(
+    embeddings = cell.embeddings,
+    loadings = feature.loadings,
+    key = reduction.key,
+    assay = assay,
+    misc = svd.lsi
+  )
+  return(reduction.data)
+}
+
+#' @param features Features to compute SLSI on. If NULL, SLSI will be run
+#' using the variable features for the Assay.
+#'
+#' @rdname RunSLSI
+#' @concept dimensional_reduction
+#' @export
+#' @method RunSLSI Assay
+#'
+RunSLSI.Assay <- function(
+  object,
+  assay = NULL,
+  features = NULL,
+  n = 50,
+  reduction.key = "SLSI_",
+  graph = NULL,
+  verbose = TRUE,
+  seed.use = 42,
+  ...
+) {
+  data.use <- PrepDR(
+    object = object,
+    features = features,
+    slot = "data",
+    verbose = verbose
+  )
+  reduction.data <- RunSLSI(
+    object = data.use,
+    assay = assay,
+    n = n,
+    reduction.key = reduction.key,
+    graph = graph,
+    verbose = verbose,
+    seed.use = seed.use,
+    ...
+  )
+  return(reduction.data)
+}
+
+#' @param reduction.name dimensional reduction name
+#' @rdname RunSLSI
+#' @concept dimensional_reduction
+#' @export
+#' @method RunSLSI Seurat
+#'
+RunSLSI.Seurat <- function(
+  object,
+  assay = NULL,
+  features = NULL,
+  n = 50,
+  reduction.name = "slsi",
+  reduction.key = "SLSI_",
+  graph = NULL,
+  verbose = TRUE,
+  seed.use = 42,
+  ...
+) {
+  assay <- assay %||% DefaultAssay(object = object)
+  assay.data <- GetAssay(object = object, assay = assay)
+  if (is.null(x = graph)) {
+    stop("Graph is not provided")
+  } else if (is.character(x = graph)) {
+    graph <- object[[graph]]
+  }
+  reduction.data <- RunSLSI(
+    object = assay.data,
+    assay = assay,
+    features = features,
+    n = n,
+    reduction.name = reduction.name,
+    reduction.key = reduction.key,
+    graph = graph,
+    verbose = verbose,
+    seed.use = seed.use,
+    ...
+  )
+  object[[reduction.name]] <- reduction.data
+  object <- LogSeuratCommand(object = object)
+  return(object)
+}
